==> react-app-logic-15.5.0.tgz: package/src/exchange/forms/useCreatePoolForm.ts <==
import { useCallback, useMemo, useState, useEffect } from 'react';
import BigNumber from 'bignumber.js';
import { DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useAssetBalance, useDetailedBalance, useFixedAmountInput } from '@onomy/react-hub';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import { OnexClient } from '@onomy/client';
import { useOnexClient } from '../hooks';
import { usePriceInputs } from './usePriceInputs';

export function useCreatePoolForm({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState<null | Awaited<ReturnType<OnexClient['createPool']>>>(
    null
  );

  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);

  const {
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePricePerBase,
    inversePricePerBaseStatus,

    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,

    reset: resetPriceInputs,
  } = usePriceInputs({ baseAsset, quoteAsset, updateOtherAsset: false });

  const setAmountBasePercentOfBalance = useCallback(
    (percentage: number) => {
      const bnBalance = new BigNumber(baseBalance.amount.toString()).times(percentage / 100);
      setAmountBaseFixed(bnBalance.toFixed(0));
    },
    [baseBalance.amount, setAmountBaseFixed]
  );

  const setAmountQuotePercentOfBalance = useCallback(
    (percentage: number) => {
      const bnBalance = new BigNumber(quoteBalance.amount.toString()).times(percentage / 100);
      setAmountQuoteFixed(bnBalance.toFixed(0));
    },
    [quoteBalance.amount, setAmountQuoteFixed]
  );

  const [amountBaseInput, setAmountBaseInput] = useFixedAmountInput({
    value: amountBaseFixed,
    onChangeValue: setAmountBaseFixed,
    meta: baseAsset.meta,
  });

  const [amountQuoteInput, setAmountQuoteInput] = useFixedAmountInput({
    value: amountQuoteFixed,
    onChangeValue: setAmountQuoteFixed,
    meta: baseAsset.meta,
  });

  const [amountQuote, amountQuoteStatus] = useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = useDetailedBalance(baseAsset, amountBaseFixed);
  const loadingStatus = useAsyncStatuses([balanceStatus]);

  const inputParams: Parameters<OnexClient['createPool']>[0] = useMemo(() => {
    return {
      coinA: baseAsset.id,
      coinB: quoteAsset.id,
      coinAmountA: amountBaseFixed,
      coinAmountB: amountQuoteFixed,
    };
  }, [baseAsset.id, quoteAsset.id, amountBaseFixed, amountQuoteFixed]);

  const validationErrors = useMemo(() => {
    const errors: {
      price?: string;
      baseAsset?: string;
      quoteAsset?: string;
      amountBaseInput?: string;
      amountQuoteInput?: string;
    } = {};

    const bnPrice = new BigNumber(price);
    const bnAmountA = new BigNumber(inputParams.coinAmountA);
    const bnAmountB = new BigNumber(inputParams.coinAmountB);

    if (!inputParams.coinA) errors.baseAsset = 'Base asset must be set';
    if (!inputParams.coinB) errors.quoteAsset = 'Quote asset must be set';

    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = 'Must be a number > 0';
    }

    if (bnAmountA.isNaN() || bnAmountA.lte(0)) {
      errors.amountBaseInput = 'Amount must be a number > 0';
    }

    if (bnAmountB.isNaN() || bnAmountB.lte(0)) {
      errors.amountQuoteInput = 'Amount must be a number > 0';
    }

    if (bnAmountA.gt(baseBalance.amount.toString())) {
      errors.amountBaseInput = 'Exceeds available balance';
    }

    if (bnAmountB.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteInput = 'Exceeds available balance';
    }

    return Object.keys(errors).length > 0 ? errors : null;
  }, [price, inputParams, baseBalance.amount, quoteBalance.amount]);

  const canSubmit = !validationErrors && !loadingStatus.pending;

  const submitTx = useCallback(
    (e?: React.FormEvent) => {
      e?.preventDefault();
      if (!canSubmit) return Promise.resolve();
      resetTxStatus();
      setTxResult(null);
      return attemptTx(async () => setTxResult((await onex?.createPool(inputParams)) ?? null));
    },
    [onex, inputParams, canSubmit, resetTxStatus, attemptTx]
  );

  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
    resetPriceInputs();
  }, [resetPriceInputs, resetTxStatus]);

  useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);

  return {
    txStatus,
    txResult,

    loadingStatus,
    balanceStatus,
    pricePerBaseStatus,
    inversePricePerBaseStatus,
    amountQuoteStatus,
    amountBaseStatus,

    baseAsset,
    quoteAsset,

    price,
    inversePrice,
    amountBaseInput,
    amountQuoteInput,

    validationErrors,
    canSubmit,

    baseBalance,
    quoteBalance,
    pricePerBase,
    pricePerQuote: inversePricePerBase,
    amountQuote,
    amountBase,

    setPrice,
    setAmountBaseInput,
    setAmountQuoteInput,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,

    submitTx,
    reset,
  };
}

export type CreatePoolFormState = ReturnType<typeof useCreatePoolForm>;

==> react-app-logic-17.5.2.tgz: package/src/exchange/forms/useCreatePoolForm.ts <==
import { useCallback, useMemo, useState, useEffect } from 'react';
import BigNumber from 'bignumber.js';
import { DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useAssetBalanceUnlocked, useDetailedBalance, useFixedAmountInput } from '@onomy/react-hub';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import { OnexClient } from '@onomy/client';
import { useOnexClient } from '../hooks';
import { usePriceInputs } from './usePriceInputs';

export function useCreatePoolForm({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [txResult, setTxResult] = useState<null | Awaited<ReturnType<OnexClient['createPool']>>>(
    null
  );

  const [baseBalance, baseBalanceStatus] = useAssetBalanceUnlocked(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalanceUnlocked(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);

  const {
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePricePerBase,
    inversePricePerBaseStatus,

    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,

    reset: resetPriceInputs,
  } = usePriceInputs({ baseAsset, quoteAsset, updateOtherAsset: false });

  const setAmountBasePercentOfBalance = useCallback(
    (percentage: number) => {
      const bnBalance = new BigNumber(baseBalance.amount.toString()).times(percentage / 100);
      setAmountBaseFixed(bnBalance.toFixed(0));
    },
    [baseBalance.amount, setAmountBaseFixed]
  );

  const setAmountQuotePercentOfBalance = useCallback(
    (percentage: number) => {
      const bnBalance = new BigNumber(quoteBalance.amount.toString()).times(percentage / 100);
      setAmountQuoteFixed(bnBalance.toFixed(0));
    },
    [quoteBalance.amount, setAmountQuoteFixed]
  );

  const [amountBaseInput, setAmountBaseInput] = useFixedAmountInput({
    value: amountBaseFixed,
    onChangeValue: setAmountBaseFixed,
    meta: baseAsset.meta,
  });

  const [amountQuoteInput, setAmountQuoteInput] = useFixedAmountInput({
    value: amountQuoteFixed,
    onChangeValue: setAmountQuoteFixed,
    meta: baseAsset.meta,
  });

  const [amountQuote, amountQuoteStatus] = useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = useDetailedBalance(baseAsset, amountBaseFixed);
  const loadingStatus = useAsyncStatuses([balanceStatus]);

  const inputParams: Parameters<OnexClient['createPool']>[0] = useMemo(() => {
    return {
      coinA: baseAsset.id,
      coinB: quoteAsset.id,
      coinAmountA: amountBaseFixed,
      coinAmountB: amountQuoteFixed,
    };
  }, [baseAsset.id, quoteAsset.id, amountBaseFixed, amountQuoteFixed]);

  const validationErrors = useMemo(() => {
    const errors: {
      price?: string;
      baseAsset?: string;
      quoteAsset?: string;
      amountBaseInput?: string;
      amountQuoteInput?: string;
    } = {};

    const bnPrice = new BigNumber(price);
    const bnAmountA = new BigNumber(inputParams.coinAmountA);
    const bnAmountB = new BigNumber(inputParams.coinAmountB);

    if (!inputParams.coinA) errors.baseAsset = 'Base asset must be set';
    if (!inputParams.coinB) errors.quoteAsset = 'Quote asset must be set';

    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = 'Must be a number > 0';
    }

    if (bnAmountA.isNaN() || bnAmountA.lte(0)) {
      errors.amountBaseInput = 'Amount must be a number > 0';
    }

    if (bnAmountB.isNaN() || bnAmountB.lte(0)) {
      errors.amountQuoteInput = 'Amount must be a number > 0';
    }

    if (bnAmountA.gt(baseBalance.amount.toString())) {
      errors.amountBaseInput = 'Exceeds available balance';
    }

    if (bnAmountB.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteInput = 'Exceeds available balance';
    }

    return Object.keys(errors).length > 0 ? errors : null;
  }, [price, inputParams, baseBalance.amount, quoteBalance.amount]);

  const canSubmit = !validationErrors && !loadingStatus.pending;

  const submitTx = useCallback(
    (e?: React.FormEvent) => {
      e?.preventDefault();
      if (!canSubmit) return Promise.resolve();
      resetTxStatus();
      setTxResult(null);
      return attemptTx(async () => setTxResult((await onex?.createPool(inputParams)) ?? null));
    },
    [onex, inputParams, canSubmit, resetTxStatus, attemptTx]
  );

  const reset = useCallback(() => {
    resetTxStatus();
    setTxResult(null);
    resetPriceInputs();
  }, [resetPriceInputs, resetTxStatus]);

  useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);

  return {
    txStatus,
    txResult,

    loadingStatus,
    balanceStatus,
    pricePerBaseStatus,
    inversePricePerBaseStatus,
    amountQuoteStatus,
    amountBaseStatus,

    baseAsset,
    quoteAsset,

    price,
    inversePrice,
    amountBaseInput,
    amountQuoteInput,

    validationErrors,
    canSubmit,

    baseBalance,
    quoteBalance,
    pricePerBase,
    pricePerQuote: inversePricePerBase,
    amountQuote,
    amountBase,

    setPrice,
    setAmountBaseInput,
    setAmountQuoteInput,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,

    submitTx,
    reset,
  };
}

export type CreatePoolFormState = ReturnType<typeof useCreatePoolForm>;
